##
# This code was generated by
# \ / _    _  _|   _  _
#  | (_)\/(_)(_|\/| |(/_  v1.0.0
#       /       /
#
# frozen_string_literal: true

module Twilio
  module REST
    class Chat < Domain
      class V1 < Version
        class ServiceContext < InstanceContext
          class ChannelContext < InstanceContext
            class MemberList < ListResource
              ##
              # Initialize the MemberList
              # @param [Version] version Version that contains the resource
              # @param [String] service_sid The SID of the
              #   {Service}[https://www.twilio.com/docs/api/chat/rest/services] the resource is
              #   associated with.
              # @param [String] channel_sid The unique ID of the
              #   {Channel}[https://www.twilio.com/docs/api/chat/rest/channels] for the member.
              # @return [MemberList] MemberList
              def initialize(version, service_sid: nil, channel_sid: nil)
                super(version)

                # Path Solution
                @solution = {service_sid: service_sid, channel_sid: channel_sid}
                @uri = "/Services/#{@solution[:service_sid]}/Channels/#{@solution[:channel_sid]}/Members"
              end

              ##
              # Create the MemberInstance
              # @param [String] identity The `identity` value that uniquely identifies the new
              #   resource's {User}[https://www.twilio.com/docs/api/chat/rest/v1/user] within the
              #   {Service}[https://www.twilio.com/docs/api/chat/rest/services]. See {access
              #   tokens}[https://www.twilio.com/docs/api/chat/guides/create-tokens] for more
              #   details.
              # @param [String] role_sid The SID of the
              #   {Role}[https://www.twilio.com/docs/api/chat/rest/roles] to assign to the member.
              #   The default roles are those specified on the
              #   {Service}[https://www.twilio.com/docs/chat/api/services].
              # @return [MemberInstance] Created MemberInstance
              def create(identity: nil, role_sid: :unset)
                data = Twilio::Values.of({'Identity' => identity, 'RoleSid' => role_sid, })

                payload = @version.create('POST', @uri, data: data)

                MemberInstance.new(
                    @version,
                    payload,
                    service_sid: @solution[:service_sid],
                    channel_sid: @solution[:channel_sid],
                )
              end

              ##
              # Lists MemberInstance records from the API as a list.
              # Unlike stream(), this operation is eager and will load `limit` records into
              # memory before returning.
              # @param [Array[String]] identity The
              #   {User}[https://www.twilio.com/docs/api/chat/rest/v1/user]'s `identity` value of
              #   the resources to read. See {access
              #   tokens}[https://www.twilio.com/docs/api/chat/guides/create-tokens] for more
              #   details.
              # @param [Integer] limit Upper limit for the number of records to return. stream()
              #    guarantees to never return more than limit.  Default is no limit
              # @param [Integer] page_size Number of records to fetch per request, when
              #    not set will use the default value of 50 records.  If no page_size is defined
              #    but a limit is defined, stream() will attempt to read the limit with the most
              #    efficient page size, i.e. min(limit, 1000)
              # @return [Array] Array of up to limit results
              def list(identity: :unset, limit: nil, page_size: nil)
                self.stream(identity: identity, limit: limit, page_size: page_size).entries
              end

              ##
              # Streams MemberInstance records from the API as an Enumerable.
              # This operation lazily loads records as efficiently as possible until the limit
              # is reached.
              # @param [Array[String]] identity The
              #   {User}[https://www.twilio.com/docs/api/chat/rest/v1/user]'s `identity` value of
              #   the resources to read. See {access
              #   tokens}[https://www.twilio.com/docs/api/chat/guides/create-tokens] for more
              #   details.
              # @param [Integer] limit Upper limit for the number of records to return. stream()
              #    guarantees to never return more than limit. Default is no limit.
              # @param [Integer] page_size Number of records to fetch per request, when
              #    not set will use the default value of 50 records. If no page_size is defined
              #    but a limit is defined, stream() will attempt to read the limit with the most
              #    efficient page size, i.e. min(limit, 1000)
              # @return [Enumerable] Enumerable that will yield up to limit results
              def stream(identity: :unset, limit: nil, page_size: nil)
                limits = @version.read_limits(limit, page_size)

                page = self.page(identity: identity, page_size: limits[:page_size], )

                @version.stream(page, limit: limits[:limit], page_limit: limits[:page_limit])
              end

              ##
              # When passed a block, yields MemberInstance records from the API.
              # This operation lazily loads records as efficiently as possible until the limit
              # is reached.
              def each
                limits = @version.read_limits

                page = self.page(page_size: limits[:page_size], )

                @version.stream(page,
                                limit: limits[:limit],
                                page_limit: limits[:page_limit]).each {|x| yield x}
              end

              ##
              # Retrieve a single page of MemberInstance records from the API.
              # Request is executed immediately.
              # @param [Array[String]] identity The
              #   {User}[https://www.twilio.com/docs/api/chat/rest/v1/user]'s `identity` value of
              #   the resources to read. See {access
              #   tokens}[https://www.twilio.com/docs/api/chat/guides/create-tokens] for more
              #   details.
              # @param [String] page_token PageToken provided by the API
              # @param [Integer] page_number Page Number, this value is simply for client state
              # @param [Integer] page_size Number of records to return, defaults to 50
              # @return [Page] Page of MemberInstance
              def page(identity: :unset, page_token: :unset, page_number: :unset, page_size: :unset)
                params = Twilio::Values.of({
                    'Identity' => Twilio.serialize_list(identity) { |e| e },
                    'PageToken' => page_token,
                    'Page' => page_number,
                    'PageSize' => page_size,
                })

                response = @version.page('GET', @uri, params: params)

                MemberPage.new(@version, response, @solution)
              end

              ##
              # Retrieve a single page of MemberInstance records from the API.
              # Request is executed immediately.
              # @param [String] target_url API-generated URL for the requested results page
              # @return [Page] Page of MemberInstance
              def get_page(target_url)
                response = @version.domain.request(
                    'GET',
                    target_url
                )
                MemberPage.new(@version, response, @solution)
              end

              ##
              # Provide a user friendly representation
              def to_s
                '#<Twilio.Chat.V1.MemberList>'
              end
            end

            class MemberPage < Page
              ##
              # Initialize the MemberPage
              # @param [Version] version Version that contains the resource
              # @param [Response] response Response from the API
              # @param [Hash] solution Path solution for the resource
              # @return [MemberPage] MemberPage
              def initialize(version, response, solution)
                super(version, response)

                # Path Solution
                @solution = solution
              end

              ##
              # Build an instance of MemberInstance
              # @param [Hash] payload Payload response from the API
              # @return [MemberInstance] MemberInstance
              def get_instance(payload)
                MemberInstance.new(
                    @version,
                    payload,
                    service_sid: @solution[:service_sid],
                    channel_sid: @solution[:channel_sid],
                )
              end

              ##
              # Provide a user friendly representation
              def to_s
                '<Twilio.Chat.V1.MemberPage>'
              end
            end

            class MemberContext < InstanceContext
              ##
              # Initialize the MemberContext
              # @param [Version] version Version that contains the resource
              # @param [String] service_sid The SID of the
              #   {Service}[https://www.twilio.com/docs/api/chat/rest/services] to fetch the
              #   resource from.
              # @param [String] channel_sid The unique ID of the
              #   {Channel}[https://www.twilio.com/docs/api/chat/rest/channels] the member to
              #   fetch belongs to. Can be the Channel resource's `sid` or `unique_name` value.
              # @param [String] sid The Twilio-provided string that uniquely identifies the
              #   Member resource to fetch.
              # @return [MemberContext] MemberContext
              def initialize(version, service_sid, channel_sid, sid)
                super(version)

                # Path Solution
                @solution = {service_sid: service_sid, channel_sid: channel_sid, sid: sid, }
                @uri = "/Services/#{@solution[:service_sid]}/Channels/#{@solution[:channel_sid]}/Members/#{@solution[:sid]}"
              end

              ##
              # Fetch the MemberInstance
              # @return [MemberInstance] Fetched MemberInstance
              def fetch
                payload = @version.fetch('GET', @uri)

                MemberInstance.new(
                    @version,
                    payload,
                    service_sid: @solution[:service_sid],
                    channel_sid: @solution[:channel_sid],
                    sid: @solution[:sid],
                )
              end

              ##
              # Delete the MemberInstance
              # @return [Boolean] true if delete succeeds, false otherwise
              def delete
                 @version.delete('DELETE', @uri)
              end

              ##
              # Update the MemberInstance
              # @param [String] role_sid The SID of the
              #   {Role}[https://www.twilio.com/docs/api/chat/rest/roles] to assign to the member.
              #   The default roles are those specified on the
              #   {Service}[https://www.twilio.com/docs/chat/api/services].
              # @param [String] last_consumed_message_index The index of the last
              #   {Message}[https://www.twilio.com/docs/api/chat/rest/messages] that the Member
              #   has read within the
              #   {Channel}[https://www.twilio.com/docs/api/chat/rest/channels].
              # @return [MemberInstance] Updated MemberInstance
              def update(role_sid: :unset, last_consumed_message_index: :unset)
                data = Twilio::Values.of({
                    'RoleSid' => role_sid,
                    'LastConsumedMessageIndex' => last_consumed_message_index,
                })

                payload = @version.update('POST', @uri, data: data)

                MemberInstance.new(
                    @version,
                    payload,
                    service_sid: @solution[:service_sid],
                    channel_sid: @solution[:channel_sid],
                    sid: @solution[:sid],
                )
              end

              ##
              # Provide a user friendly representation
              def to_s
                context = @solution.map {|k, v| "#{k}: #{v}"}.join(',')
                "#<Twilio.Chat.V1.MemberContext #{context}>"
              end

              ##
              # Provide a detailed, user friendly representation
              def inspect
                context = @solution.map {|k, v| "#{k}: #{v}"}.join(',')
                "#<Twilio.Chat.V1.MemberContext #{context}>"
              end
            end

            class MemberInstance < InstanceResource
              ##
              # Initialize the MemberInstance
              # @param [Version] version Version that contains the resource
              # @param [Hash] payload payload that contains response from Twilio
              # @param [String] service_sid The SID of the
              #   {Service}[https://www.twilio.com/docs/api/chat/rest/services] the resource is
              #   associated with.
              # @param [String] channel_sid The unique ID of the
              #   {Channel}[https://www.twilio.com/docs/api/chat/rest/channels] for the member.
              # @param [String] sid The Twilio-provided string that uniquely identifies the
              #   Member resource to fetch.
              # @return [MemberInstance] MemberInstance
              def initialize(version, payload, service_sid: nil, channel_sid: nil, sid: nil)
                super(version)

                # Marshaled Properties
                @properties = {
                    'sid' => payload['sid'],
                    'account_sid' => payload['account_sid'],
                    'channel_sid' => payload['channel_sid'],
                    'service_sid' => payload['service_sid'],
                    'identity' => payload['identity'],
                    'date_created' => Twilio.deserialize_iso8601_datetime(payload['date_created']),
                    'date_updated' => Twilio.deserialize_iso8601_datetime(payload['date_updated']),
                    'role_sid' => payload['role_sid'],
                    'last_consumed_message_index' => payload['last_consumed_message_index'] == nil ? payload['last_consumed_message_index'] : payload['last_consumed_message_index'].to_i,
                    'last_consumption_timestamp' => Twilio.deserialize_iso8601_datetime(payload['last_consumption_timestamp']),
                    'url' => payload['url'],
                }

                # Context
                @instance_context = nil
                @params = {
                    'service_sid' => service_sid,
                    'channel_sid' => channel_sid,
                    'sid' => sid || @properties['sid'],
                }
              end

              ##
              # Generate an instance context for the instance, the context is capable of
              # performing various actions.  All instance actions are proxied to the context
              # @return [MemberContext] MemberContext for this MemberInstance
              def context
                unless @instance_context
                  @instance_context = MemberContext.new(
                      @version,
                      @params['service_sid'],
                      @params['channel_sid'],
                      @params['sid'],
                  )
                end
                @instance_context
              end

              ##
              # @return [String] The unique string that identifies the resource
              def sid
                @properties['sid']
              end

              ##
              # @return [String] The SID of the Account that created the resource
              def account_sid
                @properties['account_sid']
              end

              ##
              # @return [String] The unique ID of the Channel for the member
              def channel_sid
                @properties['channel_sid']
              end

              ##
              # @return [String] The SID of the Service that the resource is associated with
              def service_sid
                @properties['service_sid']
              end

              ##
              # @return [String] The string that identifies the resource's User
              def identity
                @properties['identity']
              end

              ##
              # @return [Time] The RFC 2822 date and time in GMT when the resource was created
              def date_created
                @properties['date_created']
              end

              ##
              # @return [Time] The RFC 2822 date and time in GMT when the resource was last updated
              def date_updated
                @properties['date_updated']
              end

              ##
              # @return [String] The SID of the Role assigned to the member
              def role_sid
                @properties['role_sid']
              end

              ##
              # @return [String] The index of the last Message that the Member has read within the Channel
              def last_consumed_message_index
                @properties['last_consumed_message_index']
              end

              ##
              # @return [Time] The ISO 8601 based timestamp string that represents the date-time of the last Message read event for the Member within the Channel
              def last_consumption_timestamp
                @properties['last_consumption_timestamp']
              end

              ##
              # @return [String] The absolute URL of the Member resource
              def url
                @properties['url']
              end

              ##
              # Fetch the MemberInstance
              # @return [MemberInstance] Fetched MemberInstance
              def fetch
                context.fetch
              end

              ##
              # Delete the MemberInstance
              # @return [Boolean] true if delete succeeds, false otherwise
              def delete
                context.delete
              end

              ##
              # Update the MemberInstance
              # @param [String] role_sid The SID of the
              #   {Role}[https://www.twilio.com/docs/api/chat/rest/roles] to assign to the member.
              #   The default roles are those specified on the
              #   {Service}[https://www.twilio.com/docs/chat/api/services].
              # @param [String] last_consumed_message_index The index of the last
              #   {Message}[https://www.twilio.com/docs/api/chat/rest/messages] that the Member
              #   has read within the
              #   {Channel}[https://www.twilio.com/docs/api/chat/rest/channels].
              # @return [MemberInstance] Updated MemberInstance
              def update(role_sid: :unset, last_consumed_message_index: :unset)
                context.update(role_sid: role_sid, last_consumed_message_index: last_consumed_message_index, )
              end

              ##
              # Provide a user friendly representation
              def to_s
                values = @params.map{|k, v| "#{k}: #{v}"}.join(" ")
                "<Twilio.Chat.V1.MemberInstance #{values}>"
              end

              ##
              # Provide a detailed, user friendly representation
              def inspect
                values = @properties.map{|k, v| "#{k}: #{v}"}.join(" ")
                "<Twilio.Chat.V1.MemberInstance #{values}>"
              end
            end
          end
        end
      end
    end
  end
end