##
# This code was generated by
# \ / _    _  _|   _  _
#  | (_)\/(_)(_|\/| |(/_  v1.0.0
#       /       /
#
# frozen_string_literal: true

module Twilio
  module REST
    class Preview < Domain
      class DeployedDevices < Version
        class FleetContext < InstanceContext
          ##
          # PLEASE NOTE that this class contains preview products that are subject to change. Use them with caution. If you currently do not have developer preview access, please contact help@twilio.com.
          class CertificateList < ListResource
            ##
            # Initialize the CertificateList
            # @param [Version] version Version that contains the resource
            # @param [String] fleet_sid Specifies the unique string identifier of the Fleet
            #   that the given Certificate credential belongs to.
            # @return [CertificateList] CertificateList
            def initialize(version, fleet_sid: nil)
              super(version)

              # Path Solution
              @solution = {fleet_sid: fleet_sid}
              @uri = "/Fleets/#{@solution[:fleet_sid]}/Certificates"
            end

            ##
            # Create the CertificateInstance
            # @param [String] certificate_data Provides a URL encoded representation of the
            #   public certificate in PEM format.
            # @param [String] friendly_name Provides a human readable descriptive text for
            #   this Certificate credential, up to 256 characters long.
            # @param [String] device_sid Provides the unique string identifier of an existing
            #   Device to become authenticated with this Certificate credential.
            # @return [CertificateInstance] Created CertificateInstance
            def create(certificate_data: nil, friendly_name: :unset, device_sid: :unset)
              data = Twilio::Values.of({
                  'CertificateData' => certificate_data,
                  'FriendlyName' => friendly_name,
                  'DeviceSid' => device_sid,
              })

              payload = @version.create('POST', @uri, data: data)

              CertificateInstance.new(@version, payload, fleet_sid: @solution[:fleet_sid], )
            end

            ##
            # Lists CertificateInstance records from the API as a list.
            # Unlike stream(), this operation is eager and will load `limit` records into
            # memory before returning.
            # @param [String] device_sid Filters the resulting list of Certificates by a
            #   unique string identifier of an authenticated Device.
            # @param [Integer] limit Upper limit for the number of records to return. stream()
            #    guarantees to never return more than limit.  Default is no limit
            # @param [Integer] page_size Number of records to fetch per request, when
            #    not set will use the default value of 50 records.  If no page_size is defined
            #    but a limit is defined, stream() will attempt to read the limit with the most
            #    efficient page size, i.e. min(limit, 1000)
            # @return [Array] Array of up to limit results
            def list(device_sid: :unset, limit: nil, page_size: nil)
              self.stream(device_sid: device_sid, limit: limit, page_size: page_size).entries
            end

            ##
            # Streams CertificateInstance records from the API as an Enumerable.
            # This operation lazily loads records as efficiently as possible until the limit
            # is reached.
            # @param [String] device_sid Filters the resulting list of Certificates by a
            #   unique string identifier of an authenticated Device.
            # @param [Integer] limit Upper limit for the number of records to return. stream()
            #    guarantees to never return more than limit. Default is no limit.
            # @param [Integer] page_size Number of records to fetch per request, when
            #    not set will use the default value of 50 records. If no page_size is defined
            #    but a limit is defined, stream() will attempt to read the limit with the most
            #    efficient page size, i.e. min(limit, 1000)
            # @return [Enumerable] Enumerable that will yield up to limit results
            def stream(device_sid: :unset, limit: nil, page_size: nil)
              limits = @version.read_limits(limit, page_size)

              page = self.page(device_sid: device_sid, page_size: limits[:page_size], )

              @version.stream(page, limit: limits[:limit], page_limit: limits[:page_limit])
            end

            ##
            # When passed a block, yields CertificateInstance records from the API.
            # This operation lazily loads records as efficiently as possible until the limit
            # is reached.
            def each
              limits = @version.read_limits

              page = self.page(page_size: limits[:page_size], )

              @version.stream(page,
                              limit: limits[:limit],
                              page_limit: limits[:page_limit]).each {|x| yield x}
            end

            ##
            # Retrieve a single page of CertificateInstance records from the API.
            # Request is executed immediately.
            # @param [String] device_sid Filters the resulting list of Certificates by a
            #   unique string identifier of an authenticated Device.
            # @param [String] page_token PageToken provided by the API
            # @param [Integer] page_number Page Number, this value is simply for client state
            # @param [Integer] page_size Number of records to return, defaults to 50
            # @return [Page] Page of CertificateInstance
            def page(device_sid: :unset, page_token: :unset, page_number: :unset, page_size: :unset)
              params = Twilio::Values.of({
                  'DeviceSid' => device_sid,
                  'PageToken' => page_token,
                  'Page' => page_number,
                  'PageSize' => page_size,
              })

              response = @version.page('GET', @uri, params: params)

              CertificatePage.new(@version, response, @solution)
            end

            ##
            # Retrieve a single page of CertificateInstance records from the API.
            # Request is executed immediately.
            # @param [String] target_url API-generated URL for the requested results page
            # @return [Page] Page of CertificateInstance
            def get_page(target_url)
              response = @version.domain.request(
                  'GET',
                  target_url
              )
              CertificatePage.new(@version, response, @solution)
            end

            ##
            # Provide a user friendly representation
            def to_s
              '#<Twilio.Preview.DeployedDevices.CertificateList>'
            end
          end

          ##
          # PLEASE NOTE that this class contains preview products that are subject to change. Use them with caution. If you currently do not have developer preview access, please contact help@twilio.com.
          class CertificatePage < Page
            ##
            # Initialize the CertificatePage
            # @param [Version] version Version that contains the resource
            # @param [Response] response Response from the API
            # @param [Hash] solution Path solution for the resource
            # @return [CertificatePage] CertificatePage
            def initialize(version, response, solution)
              super(version, response)

              # Path Solution
              @solution = solution
            end

            ##
            # Build an instance of CertificateInstance
            # @param [Hash] payload Payload response from the API
            # @return [CertificateInstance] CertificateInstance
            def get_instance(payload)
              CertificateInstance.new(@version, payload, fleet_sid: @solution[:fleet_sid], )
            end

            ##
            # Provide a user friendly representation
            def to_s
              '<Twilio.Preview.DeployedDevices.CertificatePage>'
            end
          end

          ##
          # PLEASE NOTE that this class contains preview products that are subject to change. Use them with caution. If you currently do not have developer preview access, please contact help@twilio.com.
          class CertificateContext < InstanceContext
            ##
            # Initialize the CertificateContext
            # @param [Version] version Version that contains the resource
            # @param [String] fleet_sid The fleet_sid
            # @param [String] sid Provides a 34 character string that uniquely identifies the
            #   requested Certificate credential resource.
            # @return [CertificateContext] CertificateContext
            def initialize(version, fleet_sid, sid)
              super(version)

              # Path Solution
              @solution = {fleet_sid: fleet_sid, sid: sid, }
              @uri = "/Fleets/#{@solution[:fleet_sid]}/Certificates/#{@solution[:sid]}"
            end

            ##
            # Fetch the CertificateInstance
            # @return [CertificateInstance] Fetched CertificateInstance
            def fetch
              payload = @version.fetch('GET', @uri)

              CertificateInstance.new(@version, payload, fleet_sid: @solution[:fleet_sid], sid: @solution[:sid], )
            end

            ##
            # Delete the CertificateInstance
            # @return [Boolean] true if delete succeeds, false otherwise
            def delete
               @version.delete('DELETE', @uri)
            end

            ##
            # Update the CertificateInstance
            # @param [String] friendly_name Provides a human readable descriptive text for
            #   this Certificate credential, up to 256 characters long.
            # @param [String] device_sid Provides the unique string identifier of an existing
            #   Device to become authenticated with this Certificate credential.
            # @return [CertificateInstance] Updated CertificateInstance
            def update(friendly_name: :unset, device_sid: :unset)
              data = Twilio::Values.of({'FriendlyName' => friendly_name, 'DeviceSid' => device_sid, })

              payload = @version.update('POST', @uri, data: data)

              CertificateInstance.new(@version, payload, fleet_sid: @solution[:fleet_sid], sid: @solution[:sid], )
            end

            ##
            # Provide a user friendly representation
            def to_s
              context = @solution.map {|k, v| "#{k}: #{v}"}.join(',')
              "#<Twilio.Preview.DeployedDevices.CertificateContext #{context}>"
            end

            ##
            # Provide a detailed, user friendly representation
            def inspect
              context = @solution.map {|k, v| "#{k}: #{v}"}.join(',')
              "#<Twilio.Preview.DeployedDevices.CertificateContext #{context}>"
            end
          end

          ##
          # PLEASE NOTE that this class contains preview products that are subject to change. Use them with caution. If you currently do not have developer preview access, please contact help@twilio.com.
          class CertificateInstance < InstanceResource
            ##
            # Initialize the CertificateInstance
            # @param [Version] version Version that contains the resource
            # @param [Hash] payload payload that contains response from Twilio
            # @param [String] fleet_sid Specifies the unique string identifier of the Fleet
            #   that the given Certificate credential belongs to.
            # @param [String] sid Provides a 34 character string that uniquely identifies the
            #   requested Certificate credential resource.
            # @return [CertificateInstance] CertificateInstance
            def initialize(version, payload, fleet_sid: nil, sid: nil)
              super(version)

              # Marshaled Properties
              @properties = {
                  'sid' => payload['sid'],
                  'url' => payload['url'],
                  'friendly_name' => payload['friendly_name'],
                  'fleet_sid' => payload['fleet_sid'],
                  'account_sid' => payload['account_sid'],
                  'device_sid' => payload['device_sid'],
                  'thumbprint' => payload['thumbprint'],
                  'date_created' => Twilio.deserialize_iso8601_datetime(payload['date_created']),
                  'date_updated' => Twilio.deserialize_iso8601_datetime(payload['date_updated']),
              }

              # Context
              @instance_context = nil
              @params = {'fleet_sid' => fleet_sid, 'sid' => sid || @properties['sid'], }
            end

            ##
            # Generate an instance context for the instance, the context is capable of
            # performing various actions.  All instance actions are proxied to the context
            # @return [CertificateContext] CertificateContext for this CertificateInstance
            def context
              unless @instance_context
                @instance_context = CertificateContext.new(@version, @params['fleet_sid'], @params['sid'], )
              end
              @instance_context
            end

            ##
            # @return [String] A string that uniquely identifies this Certificate.
            def sid
              @properties['sid']
            end

            ##
            # @return [String] URL of this Certificate.
            def url
              @properties['url']
            end

            ##
            # @return [String] A human readable description for this Certificate.
            def friendly_name
              @properties['friendly_name']
            end

            ##
            # @return [String] The unique identifier of the Fleet.
            def fleet_sid
              @properties['fleet_sid']
            end

            ##
            # @return [String] The unique SID that identifies this Account.
            def account_sid
              @properties['account_sid']
            end

            ##
            # @return [String] The unique identifier of a mapped Device.
            def device_sid
              @properties['device_sid']
            end

            ##
            # @return [String] A Certificate unique payload hash.
            def thumbprint
              @properties['thumbprint']
            end

            ##
            # @return [Time] The date this Certificate was created.
            def date_created
              @properties['date_created']
            end

            ##
            # @return [Time] The date this Certificate was updated.
            def date_updated
              @properties['date_updated']
            end

            ##
            # Fetch the CertificateInstance
            # @return [CertificateInstance] Fetched CertificateInstance
            def fetch
              context.fetch
            end

            ##
            # Delete the CertificateInstance
            # @return [Boolean] true if delete succeeds, false otherwise
            def delete
              context.delete
            end

            ##
            # Update the CertificateInstance
            # @param [String] friendly_name Provides a human readable descriptive text for
            #   this Certificate credential, up to 256 characters long.
            # @param [String] device_sid Provides the unique string identifier of an existing
            #   Device to become authenticated with this Certificate credential.
            # @return [CertificateInstance] Updated CertificateInstance
            def update(friendly_name: :unset, device_sid: :unset)
              context.update(friendly_name: friendly_name, device_sid: device_sid, )
            end

            ##
            # Provide a user friendly representation
            def to_s
              values = @params.map{|k, v| "#{k}: #{v}"}.join(" ")
              "<Twilio.Preview.DeployedDevices.CertificateInstance #{values}>"
            end

            ##
            # Provide a detailed, user friendly representation
            def inspect
              values = @properties.map{|k, v| "#{k}: #{v}"}.join(" ")
              "<Twilio.Preview.DeployedDevices.CertificateInstance #{values}>"
            end
          end
        end
      end
    end
  end
end