##
# This code was generated by
# \ / _    _  _|   _  _
#  | (_)\/(_)(_|\/| |(/_  v1.0.0
#       /       /
#
# frozen_string_literal: true

module Twilio
  module REST
    class Supersim < Domain
      class V1 < Version
        class NetworkAccessProfileContext < InstanceContext
          ##
          # PLEASE NOTE that this class contains beta products that are subject to change. Use them with caution.
          class NetworkAccessProfileNetworkList < ListResource
            ##
            # Initialize the NetworkAccessProfileNetworkList
            # @param [Version] version Version that contains the resource
            # @param [String] network_access_profile_sid The unique string that identifies the
            #   Network resource's Network Access Profile resource.
            # @return [NetworkAccessProfileNetworkList] NetworkAccessProfileNetworkList
            def initialize(version, network_access_profile_sid: nil)
              super(version)

              # Path Solution
              @solution = {network_access_profile_sid: network_access_profile_sid}
              @uri = "/NetworkAccessProfiles/#{@solution[:network_access_profile_sid]}/Networks"
            end

            ##
            # Lists NetworkAccessProfileNetworkInstance records from the API as a list.
            # Unlike stream(), this operation is eager and will load `limit` records into
            # memory before returning.
            # @param [Integer] limit Upper limit for the number of records to return. stream()
            #    guarantees to never return more than limit.  Default is no limit
            # @param [Integer] page_size Number of records to fetch per request, when
            #    not set will use the default value of 50 records.  If no page_size is defined
            #    but a limit is defined, stream() will attempt to read the limit with the most
            #    efficient page size, i.e. min(limit, 1000)
            # @return [Array] Array of up to limit results
            def list(limit: nil, page_size: nil)
              self.stream(limit: limit, page_size: page_size).entries
            end

            ##
            # Streams NetworkAccessProfileNetworkInstance records from the API as an Enumerable.
            # This operation lazily loads records as efficiently as possible until the limit
            # is reached.
            # @param [Integer] limit Upper limit for the number of records to return. stream()
            #    guarantees to never return more than limit. Default is no limit.
            # @param [Integer] page_size Number of records to fetch per request, when
            #    not set will use the default value of 50 records. If no page_size is defined
            #    but a limit is defined, stream() will attempt to read the limit with the most
            #    efficient page size, i.e. min(limit, 1000)
            # @return [Enumerable] Enumerable that will yield up to limit results
            def stream(limit: nil, page_size: nil)
              limits = @version.read_limits(limit, page_size)

              page = self.page(page_size: limits[:page_size], )

              @version.stream(page, limit: limits[:limit], page_limit: limits[:page_limit])
            end

            ##
            # When passed a block, yields NetworkAccessProfileNetworkInstance records from the API.
            # This operation lazily loads records as efficiently as possible until the limit
            # is reached.
            def each
              limits = @version.read_limits

              page = self.page(page_size: limits[:page_size], )

              @version.stream(page,
                              limit: limits[:limit],
                              page_limit: limits[:page_limit]).each {|x| yield x}
            end

            ##
            # Retrieve a single page of NetworkAccessProfileNetworkInstance records from the API.
            # Request is executed immediately.
            # @param [String] page_token PageToken provided by the API
            # @param [Integer] page_number Page Number, this value is simply for client state
            # @param [Integer] page_size Number of records to return, defaults to 50
            # @return [Page] Page of NetworkAccessProfileNetworkInstance
            def page(page_token: :unset, page_number: :unset, page_size: :unset)
              params = Twilio::Values.of({
                  'PageToken' => page_token,
                  'Page' => page_number,
                  'PageSize' => page_size,
              })

              response = @version.page('GET', @uri, params: params)

              NetworkAccessProfileNetworkPage.new(@version, response, @solution)
            end

            ##
            # Retrieve a single page of NetworkAccessProfileNetworkInstance records from the API.
            # Request is executed immediately.
            # @param [String] target_url API-generated URL for the requested results page
            # @return [Page] Page of NetworkAccessProfileNetworkInstance
            def get_page(target_url)
              response = @version.domain.request(
                  'GET',
                  target_url
              )
              NetworkAccessProfileNetworkPage.new(@version, response, @solution)
            end

            ##
            # Create the NetworkAccessProfileNetworkInstance
            # @param [String] network The SID of the Network resource to be added to the
            #   Network Access Profile resource.
            # @return [NetworkAccessProfileNetworkInstance] Created NetworkAccessProfileNetworkInstance
            def create(network: nil)
              data = Twilio::Values.of({'Network' => network, })

              payload = @version.create('POST', @uri, data: data)

              NetworkAccessProfileNetworkInstance.new(
                  @version,
                  payload,
                  network_access_profile_sid: @solution[:network_access_profile_sid],
              )
            end

            ##
            # Provide a user friendly representation
            def to_s
              '#<Twilio.Supersim.V1.NetworkAccessProfileNetworkList>'
            end
          end

          ##
          # PLEASE NOTE that this class contains beta products that are subject to change. Use them with caution.
          class NetworkAccessProfileNetworkPage < Page
            ##
            # Initialize the NetworkAccessProfileNetworkPage
            # @param [Version] version Version that contains the resource
            # @param [Response] response Response from the API
            # @param [Hash] solution Path solution for the resource
            # @return [NetworkAccessProfileNetworkPage] NetworkAccessProfileNetworkPage
            def initialize(version, response, solution)
              super(version, response)

              # Path Solution
              @solution = solution
            end

            ##
            # Build an instance of NetworkAccessProfileNetworkInstance
            # @param [Hash] payload Payload response from the API
            # @return [NetworkAccessProfileNetworkInstance] NetworkAccessProfileNetworkInstance
            def get_instance(payload)
              NetworkAccessProfileNetworkInstance.new(
                  @version,
                  payload,
                  network_access_profile_sid: @solution[:network_access_profile_sid],
              )
            end

            ##
            # Provide a user friendly representation
            def to_s
              '<Twilio.Supersim.V1.NetworkAccessProfileNetworkPage>'
            end
          end

          ##
          # PLEASE NOTE that this class contains beta products that are subject to change. Use them with caution.
          class NetworkAccessProfileNetworkContext < InstanceContext
            ##
            # Initialize the NetworkAccessProfileNetworkContext
            # @param [Version] version Version that contains the resource
            # @param [String] network_access_profile_sid The unique string that identifies the
            #   Network Access Profile resource.
            # @param [String] sid The SID of the Network resource to fetch.
            # @return [NetworkAccessProfileNetworkContext] NetworkAccessProfileNetworkContext
            def initialize(version, network_access_profile_sid, sid)
              super(version)

              # Path Solution
              @solution = {network_access_profile_sid: network_access_profile_sid, sid: sid, }
              @uri = "/NetworkAccessProfiles/#{@solution[:network_access_profile_sid]}/Networks/#{@solution[:sid]}"
            end

            ##
            # Delete the NetworkAccessProfileNetworkInstance
            # @return [Boolean] true if delete succeeds, false otherwise
            def delete
               @version.delete('DELETE', @uri)
            end

            ##
            # Fetch the NetworkAccessProfileNetworkInstance
            # @return [NetworkAccessProfileNetworkInstance] Fetched NetworkAccessProfileNetworkInstance
            def fetch
              payload = @version.fetch('GET', @uri)

              NetworkAccessProfileNetworkInstance.new(
                  @version,
                  payload,
                  network_access_profile_sid: @solution[:network_access_profile_sid],
                  sid: @solution[:sid],
              )
            end

            ##
            # Provide a user friendly representation
            def to_s
              context = @solution.map {|k, v| "#{k}: #{v}"}.join(',')
              "#<Twilio.Supersim.V1.NetworkAccessProfileNetworkContext #{context}>"
            end

            ##
            # Provide a detailed, user friendly representation
            def inspect
              context = @solution.map {|k, v| "#{k}: #{v}"}.join(',')
              "#<Twilio.Supersim.V1.NetworkAccessProfileNetworkContext #{context}>"
            end
          end

          ##
          # PLEASE NOTE that this class contains beta products that are subject to change. Use them with caution.
          class NetworkAccessProfileNetworkInstance < InstanceResource
            ##
            # Initialize the NetworkAccessProfileNetworkInstance
            # @param [Version] version Version that contains the resource
            # @param [Hash] payload payload that contains response from Twilio
            # @param [String] network_access_profile_sid The unique string that identifies the
            #   Network resource's Network Access Profile resource.
            # @param [String] sid The SID of the Network resource to fetch.
            # @return [NetworkAccessProfileNetworkInstance] NetworkAccessProfileNetworkInstance
            def initialize(version, payload, network_access_profile_sid: nil, sid: nil)
              super(version)

              # Marshaled Properties
              @properties = {
                  'sid' => payload['sid'],
                  'network_access_profile_sid' => payload['network_access_profile_sid'],
                  'friendly_name' => payload['friendly_name'],
                  'iso_country' => payload['iso_country'],
                  'identifiers' => payload['identifiers'],
                  'url' => payload['url'],
              }

              # Context
              @instance_context = nil
              @params = {
                  'network_access_profile_sid' => network_access_profile_sid,
                  'sid' => sid || @properties['sid'],
              }
            end

            ##
            # Generate an instance context for the instance, the context is capable of
            # performing various actions.  All instance actions are proxied to the context
            # @return [NetworkAccessProfileNetworkContext] NetworkAccessProfileNetworkContext for this NetworkAccessProfileNetworkInstance
            def context
              unless @instance_context
                @instance_context = NetworkAccessProfileNetworkContext.new(
                    @version,
                    @params['network_access_profile_sid'],
                    @params['sid'],
                )
              end
              @instance_context
            end

            ##
            # @return [String] The unique string that identifies the resource
            def sid
              @properties['sid']
            end

            ##
            # @return [String] The unique string that identifies the Network Access Profile resource
            def network_access_profile_sid
              @properties['network_access_profile_sid']
            end

            ##
            # @return [String] A human readable identifier of this resource
            def friendly_name
              @properties['friendly_name']
            end

            ##
            # @return [String] The ISO country code of the Network resource
            def iso_country
              @properties['iso_country']
            end

            ##
            # @return [Array[Hash]] The MCC/MNCs included in the resource
            def identifiers
              @properties['identifiers']
            end

            ##
            # @return [String] The absolute URL of the resource
            def url
              @properties['url']
            end

            ##
            # Delete the NetworkAccessProfileNetworkInstance
            # @return [Boolean] true if delete succeeds, false otherwise
            def delete
              context.delete
            end

            ##
            # Fetch the NetworkAccessProfileNetworkInstance
            # @return [NetworkAccessProfileNetworkInstance] Fetched NetworkAccessProfileNetworkInstance
            def fetch
              context.fetch
            end

            ##
            # Provide a user friendly representation
            def to_s
              values = @params.map{|k, v| "#{k}: #{v}"}.join(" ")
              "<Twilio.Supersim.V1.NetworkAccessProfileNetworkInstance #{values}>"
            end

            ##
            # Provide a detailed, user friendly representation
            def inspect
              values = @properties.map{|k, v| "#{k}: #{v}"}.join(" ")
              "<Twilio.Supersim.V1.NetworkAccessProfileNetworkInstance #{values}>"
            end
          end
        end
      end
    end
  end
end