##
# This code was generated by
# \ / _    _  _|   _  _
#  | (_)\/(_)(_|\/| |(/_  v1.0.0
#       /       /
#
# frozen_string_literal: true

module Twilio
  module REST
    class Taskrouter < Domain
      class V1 < Version
        class WorkspaceContext < InstanceContext
          class WorkflowContext < InstanceContext
            class WorkflowCumulativeStatisticsList < ListResource
              ##
              # Initialize the WorkflowCumulativeStatisticsList
              # @param [Version] version Version that contains the resource
              # @param [String] workspace_sid The SID of the Workspace that contains the
              #   Workflow.
              # @param [String] workflow_sid Returns the list of Tasks that are being controlled
              #   by the Workflow with the specified Sid value.
              # @return [WorkflowCumulativeStatisticsList] WorkflowCumulativeStatisticsList
              def initialize(version, workspace_sid: nil, workflow_sid: nil)
                super(version)

                # Path Solution
                @solution = {workspace_sid: workspace_sid, workflow_sid: workflow_sid}
              end

              ##
              # Provide a user friendly representation
              def to_s
                '#<Twilio.Taskrouter.V1.WorkflowCumulativeStatisticsList>'
              end
            end

            class WorkflowCumulativeStatisticsPage < Page
              ##
              # Initialize the WorkflowCumulativeStatisticsPage
              # @param [Version] version Version that contains the resource
              # @param [Response] response Response from the API
              # @param [Hash] solution Path solution for the resource
              # @return [WorkflowCumulativeStatisticsPage] WorkflowCumulativeStatisticsPage
              def initialize(version, response, solution)
                super(version, response)

                # Path Solution
                @solution = solution
              end

              ##
              # Build an instance of WorkflowCumulativeStatisticsInstance
              # @param [Hash] payload Payload response from the API
              # @return [WorkflowCumulativeStatisticsInstance] WorkflowCumulativeStatisticsInstance
              def get_instance(payload)
                WorkflowCumulativeStatisticsInstance.new(
                    @version,
                    payload,
                    workspace_sid: @solution[:workspace_sid],
                    workflow_sid: @solution[:workflow_sid],
                )
              end

              ##
              # Provide a user friendly representation
              def to_s
                '<Twilio.Taskrouter.V1.WorkflowCumulativeStatisticsPage>'
              end
            end

            class WorkflowCumulativeStatisticsContext < InstanceContext
              ##
              # Initialize the WorkflowCumulativeStatisticsContext
              # @param [Version] version Version that contains the resource
              # @param [String] workspace_sid The SID of the Workspace with the resource to
              #   fetch.
              # @param [String] workflow_sid Returns the list of Tasks that are being controlled
              #   by the Workflow with the specified Sid value.
              # @return [WorkflowCumulativeStatisticsContext] WorkflowCumulativeStatisticsContext
              def initialize(version, workspace_sid, workflow_sid)
                super(version)

                # Path Solution
                @solution = {workspace_sid: workspace_sid, workflow_sid: workflow_sid, }
                @uri = "/Workspaces/#{@solution[:workspace_sid]}/Workflows/#{@solution[:workflow_sid]}/CumulativeStatistics"
              end

              ##
              # Fetch the WorkflowCumulativeStatisticsInstance
              # @param [Time] end_date Only include usage that occurred on or before this date,
              #   specified in GMT as an {ISO 8601}[https://en.wikipedia.org/wiki/ISO_8601]
              #   date-time.
              # @param [String] minutes Only calculate statistics since this many minutes in the
              #   past. The default 15 minutes. This is helpful for displaying statistics for the
              #   last 15 minutes, 240 minutes (4 hours), and 480 minutes (8 hours) to see trends.
              # @param [Time] start_date Only calculate statistics from this date and time and
              #   later, specified in {ISO 8601}[https://en.wikipedia.org/wiki/ISO_8601] format.
              # @param [String] task_channel Only calculate cumulative statistics on this
              #   TaskChannel. Can be the TaskChannel's SID or its `unique_name`, such as `voice`,
              #   `sms`, or `default`.
              # @param [String] split_by_wait_time A comma separated list of values that
              #   describes the thresholds, in seconds, to calculate statistics on. For each
              #   threshold specified, the number of Tasks canceled and reservations accepted
              #   above and below the specified thresholds in seconds are computed. For example,
              #   `5,30` would show splits of Tasks that were canceled or accepted before and
              #   after 5 seconds and before and after 30 seconds. This can be used to show short
              #   abandoned Tasks or Tasks that failed to meet an SLA.
              # @return [WorkflowCumulativeStatisticsInstance] Fetched WorkflowCumulativeStatisticsInstance
              def fetch(end_date: :unset, minutes: :unset, start_date: :unset, task_channel: :unset, split_by_wait_time: :unset)
                params = Twilio::Values.of({
                    'EndDate' => Twilio.serialize_iso8601_datetime(end_date),
                    'Minutes' => minutes,
                    'StartDate' => Twilio.serialize_iso8601_datetime(start_date),
                    'TaskChannel' => task_channel,
                    'SplitByWaitTime' => split_by_wait_time,
                })

                payload = @version.fetch('GET', @uri, params: params)

                WorkflowCumulativeStatisticsInstance.new(
                    @version,
                    payload,
                    workspace_sid: @solution[:workspace_sid],
                    workflow_sid: @solution[:workflow_sid],
                )
              end

              ##
              # Provide a user friendly representation
              def to_s
                context = @solution.map {|k, v| "#{k}: #{v}"}.join(',')
                "#<Twilio.Taskrouter.V1.WorkflowCumulativeStatisticsContext #{context}>"
              end

              ##
              # Provide a detailed, user friendly representation
              def inspect
                context = @solution.map {|k, v| "#{k}: #{v}"}.join(',')
                "#<Twilio.Taskrouter.V1.WorkflowCumulativeStatisticsContext #{context}>"
              end
            end

            class WorkflowCumulativeStatisticsInstance < InstanceResource
              ##
              # Initialize the WorkflowCumulativeStatisticsInstance
              # @param [Version] version Version that contains the resource
              # @param [Hash] payload payload that contains response from Twilio
              # @param [String] workspace_sid The SID of the Workspace that contains the
              #   Workflow.
              # @param [String] workflow_sid Returns the list of Tasks that are being controlled
              #   by the Workflow with the specified Sid value.
              # @return [WorkflowCumulativeStatisticsInstance] WorkflowCumulativeStatisticsInstance
              def initialize(version, payload, workspace_sid: nil, workflow_sid: nil)
                super(version)

                # Marshaled Properties
                @properties = {
                    'account_sid' => payload['account_sid'],
                    'avg_task_acceptance_time' => payload['avg_task_acceptance_time'].to_i,
                    'start_time' => Twilio.deserialize_iso8601_datetime(payload['start_time']),
                    'end_time' => Twilio.deserialize_iso8601_datetime(payload['end_time']),
                    'reservations_created' => payload['reservations_created'].to_i,
                    'reservations_accepted' => payload['reservations_accepted'].to_i,
                    'reservations_rejected' => payload['reservations_rejected'].to_i,
                    'reservations_timed_out' => payload['reservations_timed_out'].to_i,
                    'reservations_canceled' => payload['reservations_canceled'].to_i,
                    'reservations_rescinded' => payload['reservations_rescinded'].to_i,
                    'split_by_wait_time' => payload['split_by_wait_time'],
                    'wait_duration_until_accepted' => payload['wait_duration_until_accepted'],
                    'wait_duration_until_canceled' => payload['wait_duration_until_canceled'],
                    'tasks_canceled' => payload['tasks_canceled'].to_i,
                    'tasks_completed' => payload['tasks_completed'].to_i,
                    'tasks_entered' => payload['tasks_entered'].to_i,
                    'tasks_deleted' => payload['tasks_deleted'].to_i,
                    'tasks_moved' => payload['tasks_moved'].to_i,
                    'tasks_timed_out_in_workflow' => payload['tasks_timed_out_in_workflow'].to_i,
                    'workflow_sid' => payload['workflow_sid'],
                    'workspace_sid' => payload['workspace_sid'],
                    'url' => payload['url'],
                }

                # Context
                @instance_context = nil
                @params = {'workspace_sid' => workspace_sid, 'workflow_sid' => workflow_sid, }
              end

              ##
              # Generate an instance context for the instance, the context is capable of
              # performing various actions.  All instance actions are proxied to the context
              # @return [WorkflowCumulativeStatisticsContext] WorkflowCumulativeStatisticsContext for this WorkflowCumulativeStatisticsInstance
              def context
                unless @instance_context
                  @instance_context = WorkflowCumulativeStatisticsContext.new(
                      @version,
                      @params['workspace_sid'],
                      @params['workflow_sid'],
                  )
                end
                @instance_context
              end

              ##
              # @return [String] The SID of the Account that created the resource
              def account_sid
                @properties['account_sid']
              end

              ##
              # @return [String] The average time in seconds between Task creation and acceptance
              def avg_task_acceptance_time
                @properties['avg_task_acceptance_time']
              end

              ##
              # @return [Time] The beginning of the interval during which these statistics were calculated
              def start_time
                @properties['start_time']
              end

              ##
              # @return [Time] The end of the interval during which these statistics were calculated
              def end_time
                @properties['end_time']
              end

              ##
              # @return [String] The total number of Reservations that were created for Workers
              def reservations_created
                @properties['reservations_created']
              end

              ##
              # @return [String] The total number of Reservations accepted by Workers
              def reservations_accepted
                @properties['reservations_accepted']
              end

              ##
              # @return [String] The total number of Reservations that were rejected
              def reservations_rejected
                @properties['reservations_rejected']
              end

              ##
              # @return [String] The total number of Reservations that were timed out
              def reservations_timed_out
                @properties['reservations_timed_out']
              end

              ##
              # @return [String] The total number of Reservations that were canceled
              def reservations_canceled
                @properties['reservations_canceled']
              end

              ##
              # @return [String] The total number of Reservations that were rescinded
              def reservations_rescinded
                @properties['reservations_rescinded']
              end

              ##
              # @return [Hash] A list of objects that describe the Tasks canceled and reservations accepted above and below the specified thresholds
              def split_by_wait_time
                @properties['split_by_wait_time']
              end

              ##
              # @return [Hash] The wait duration statistics for Tasks that were accepted
              def wait_duration_until_accepted
                @properties['wait_duration_until_accepted']
              end

              ##
              # @return [Hash] The wait duration statistics for Tasks that were canceled
              def wait_duration_until_canceled
                @properties['wait_duration_until_canceled']
              end

              ##
              # @return [String] The total number of Tasks that were canceled
              def tasks_canceled
                @properties['tasks_canceled']
              end

              ##
              # @return [String] The total number of Tasks that were completed
              def tasks_completed
                @properties['tasks_completed']
              end

              ##
              # @return [String] The total number of Tasks that entered the Workflow
              def tasks_entered
                @properties['tasks_entered']
              end

              ##
              # @return [String] The total number of Tasks that were deleted
              def tasks_deleted
                @properties['tasks_deleted']
              end

              ##
              # @return [String] The total number of Tasks that were moved from one queue to another
              def tasks_moved
                @properties['tasks_moved']
              end

              ##
              # @return [String] The total number of Tasks that were timed out of their Workflows
              def tasks_timed_out_in_workflow
                @properties['tasks_timed_out_in_workflow']
              end

              ##
              # @return [String] Returns the list of Tasks that are being controlled by the Workflow with the specified Sid value
              def workflow_sid
                @properties['workflow_sid']
              end

              ##
              # @return [String] The SID of the Workspace that contains the Workflow.
              def workspace_sid
                @properties['workspace_sid']
              end

              ##
              # @return [String] The absolute URL of the Workflow statistics resource
              def url
                @properties['url']
              end

              ##
              # Fetch the WorkflowCumulativeStatisticsInstance
              # @param [Time] end_date Only include usage that occurred on or before this date,
              #   specified in GMT as an {ISO 8601}[https://en.wikipedia.org/wiki/ISO_8601]
              #   date-time.
              # @param [String] minutes Only calculate statistics since this many minutes in the
              #   past. The default 15 minutes. This is helpful for displaying statistics for the
              #   last 15 minutes, 240 minutes (4 hours), and 480 minutes (8 hours) to see trends.
              # @param [Time] start_date Only calculate statistics from this date and time and
              #   later, specified in {ISO 8601}[https://en.wikipedia.org/wiki/ISO_8601] format.
              # @param [String] task_channel Only calculate cumulative statistics on this
              #   TaskChannel. Can be the TaskChannel's SID or its `unique_name`, such as `voice`,
              #   `sms`, or `default`.
              # @param [String] split_by_wait_time A comma separated list of values that
              #   describes the thresholds, in seconds, to calculate statistics on. For each
              #   threshold specified, the number of Tasks canceled and reservations accepted
              #   above and below the specified thresholds in seconds are computed. For example,
              #   `5,30` would show splits of Tasks that were canceled or accepted before and
              #   after 5 seconds and before and after 30 seconds. This can be used to show short
              #   abandoned Tasks or Tasks that failed to meet an SLA.
              # @return [WorkflowCumulativeStatisticsInstance] Fetched WorkflowCumulativeStatisticsInstance
              def fetch(end_date: :unset, minutes: :unset, start_date: :unset, task_channel: :unset, split_by_wait_time: :unset)
                context.fetch(
                    end_date: end_date,
                    minutes: minutes,
                    start_date: start_date,
                    task_channel: task_channel,
                    split_by_wait_time: split_by_wait_time,
                )
              end

              ##
              # Provide a user friendly representation
              def to_s
                values = @params.map{|k, v| "#{k}: #{v}"}.join(" ")
                "<Twilio.Taskrouter.V1.WorkflowCumulativeStatisticsInstance #{values}>"
              end

              ##
              # Provide a detailed, user friendly representation
              def inspect
                values = @properties.map{|k, v| "#{k}: #{v}"}.join(" ")
                "<Twilio.Taskrouter.V1.WorkflowCumulativeStatisticsInstance #{values}>"
              end
            end
          end
        end
      end
    end
  end
end