##
# This code was generated by
# \ / _    _  _|   _  _
#  | (_)\/(_)(_|\/| |(/_  v1.0.0
#       /       /
#
# frozen_string_literal: true

module Twilio
  module REST
    class Verify < Domain
      class V2 < Version
        class VerificationTemplateList < ListResource
          ##
          # Initialize the VerificationTemplateList
          # @param [Version] version Version that contains the resource
          # @return [VerificationTemplateList] VerificationTemplateList
          def initialize(version)
            super(version)

            # Path Solution
            @solution = {}
            @uri = "/Templates"
          end

          ##
          # Lists VerificationTemplateInstance records from the API as a list.
          # Unlike stream(), this operation is eager and will load `limit` records into
          # memory before returning.
          # @param [String] friendly_name String filter used to query templates with a given
          #   friendly name
          # @param [Integer] limit Upper limit for the number of records to return. stream()
          #    guarantees to never return more than limit.  Default is no limit
          # @param [Integer] page_size Number of records to fetch per request, when
          #    not set will use the default value of 50 records.  If no page_size is defined
          #    but a limit is defined, stream() will attempt to read the limit with the most
          #    efficient page size, i.e. min(limit, 1000)
          # @return [Array] Array of up to limit results
          def list(friendly_name: :unset, limit: nil, page_size: nil)
            self.stream(friendly_name: friendly_name, limit: limit, page_size: page_size).entries
          end

          ##
          # Streams VerificationTemplateInstance records from the API as an Enumerable.
          # This operation lazily loads records as efficiently as possible until the limit
          # is reached.
          # @param [String] friendly_name String filter used to query templates with a given
          #   friendly name
          # @param [Integer] limit Upper limit for the number of records to return. stream()
          #    guarantees to never return more than limit. Default is no limit.
          # @param [Integer] page_size Number of records to fetch per request, when
          #    not set will use the default value of 50 records. If no page_size is defined
          #    but a limit is defined, stream() will attempt to read the limit with the most
          #    efficient page size, i.e. min(limit, 1000)
          # @return [Enumerable] Enumerable that will yield up to limit results
          def stream(friendly_name: :unset, limit: nil, page_size: nil)
            limits = @version.read_limits(limit, page_size)

            page = self.page(friendly_name: friendly_name, page_size: limits[:page_size], )

            @version.stream(page, limit: limits[:limit], page_limit: limits[:page_limit])
          end

          ##
          # When passed a block, yields VerificationTemplateInstance records from the API.
          # This operation lazily loads records as efficiently as possible until the limit
          # is reached.
          def each
            limits = @version.read_limits

            page = self.page(page_size: limits[:page_size], )

            @version.stream(page,
                            limit: limits[:limit],
                            page_limit: limits[:page_limit]).each {|x| yield x}
          end

          ##
          # Retrieve a single page of VerificationTemplateInstance records from the API.
          # Request is executed immediately.
          # @param [String] friendly_name String filter used to query templates with a given
          #   friendly name
          # @param [String] page_token PageToken provided by the API
          # @param [Integer] page_number Page Number, this value is simply for client state
          # @param [Integer] page_size Number of records to return, defaults to 50
          # @return [Page] Page of VerificationTemplateInstance
          def page(friendly_name: :unset, page_token: :unset, page_number: :unset, page_size: :unset)
            params = Twilio::Values.of({
                'FriendlyName' => friendly_name,
                'PageToken' => page_token,
                'Page' => page_number,
                'PageSize' => page_size,
            })

            response = @version.page('GET', @uri, params: params)

            VerificationTemplatePage.new(@version, response, @solution)
          end

          ##
          # Retrieve a single page of VerificationTemplateInstance records from the API.
          # Request is executed immediately.
          # @param [String] target_url API-generated URL for the requested results page
          # @return [Page] Page of VerificationTemplateInstance
          def get_page(target_url)
            response = @version.domain.request(
                'GET',
                target_url
            )
            VerificationTemplatePage.new(@version, response, @solution)
          end

          ##
          # Provide a user friendly representation
          def to_s
            '#<Twilio.Verify.V2.VerificationTemplateList>'
          end
        end

        class VerificationTemplatePage < Page
          ##
          # Initialize the VerificationTemplatePage
          # @param [Version] version Version that contains the resource
          # @param [Response] response Response from the API
          # @param [Hash] solution Path solution for the resource
          # @return [VerificationTemplatePage] VerificationTemplatePage
          def initialize(version, response, solution)
            super(version, response)

            # Path Solution
            @solution = solution
          end

          ##
          # Build an instance of VerificationTemplateInstance
          # @param [Hash] payload Payload response from the API
          # @return [VerificationTemplateInstance] VerificationTemplateInstance
          def get_instance(payload)
            VerificationTemplateInstance.new(@version, payload, )
          end

          ##
          # Provide a user friendly representation
          def to_s
            '<Twilio.Verify.V2.VerificationTemplatePage>'
          end
        end

        class VerificationTemplateInstance < InstanceResource
          ##
          # Initialize the VerificationTemplateInstance
          # @param [Version] version Version that contains the resource
          # @param [Hash] payload payload that contains response from Twilio
          # @return [VerificationTemplateInstance] VerificationTemplateInstance
          def initialize(version, payload)
            super(version)

            # Marshaled Properties
            @properties = {
                'sid' => payload['sid'],
                'account_sid' => payload['account_sid'],
                'friendly_name' => payload['friendly_name'],
                'translations' => payload['translations'],
            }
          end

          ##
          # @return [String] A string that uniquely identifies this Template
          def sid
            @properties['sid']
          end

          ##
          # @return [String] Account Sid
          def account_sid
            @properties['account_sid']
          end

          ##
          # @return [String] A string to describe the verification template
          def friendly_name
            @properties['friendly_name']
          end

          ##
          # @return [Hash] Ojbect with the template translations.
          def translations
            @properties['translations']
          end

          ##
          # Provide a user friendly representation
          def to_s
            "<Twilio.Verify.V2.VerificationTemplateInstance>"
          end

          ##
          # Provide a detailed, user friendly representation
          def inspect
            "<Twilio.Verify.V2.VerificationTemplateInstance>"
          end
        end
      end
    end
  end
end